Package: TAPseq
Type: Package
Title: Targeted scRNA-seq primer design for TAP-seq
Version: 1.25.0
Authors@R: c(
  person(c("Andreas", "R."), "Gschwind", email = "andreas.gschwind@stanford.edu",
         role = c("aut", "cre"), comment = c(ORCID = "0000-0002-0769-6907")),
  person("Lars", "Velten", email = "lars.velten@crg.eu", role = "aut",
         comment = c(ORCID = "0000-0002-1233-5874")),
  person(c("Lars", "M."), "Steinmetz", role = "aut"))
Description: Design primers for targeted single-cell RNA-seq used by TAP-seq. Create sequence
  templates for target gene panels and design gene-specific primers using Primer3. Potential
  off-targets can be estimated with BLAST. Requires working installations of Primer3 and BLASTn.
URL: https://github.com/argschwind/TAPseq
BugRepots: https://github.com/argschwind/TAPseq/issues
biocViews: SingleCell, Sequencing, Technology, CRISPR, PooledScreens
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.2.3
VignetteBuilder: knitr
Depends: R (>= 4.0.0)
Imports: methods, GenomicAlignments, GenomicRanges, IRanges,
        BiocGenerics, S4Vectors (>= 0.20.1), GenomeInfoDb, BSgenome,
        GenomicFeatures, Biostrings, dplyr, tidyr, BiocParallel
Suggests: testthat, BSgenome.Hsapiens.UCSC.hg38, knitr, rmarkdown,
        ggplot2, Seurat, glmnet, cowplot, Matrix, rtracklayer,
        BiocStyle
SystemRequirements: Primer3 (>= 2.5.0), BLAST+ (>=2.6.0)
git_url: https://git.bioconductor.org/packages/TAPseq
git_branch: devel
git_last_commit: ce52dd7
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:35:06 UTC; biocbuild
Author: Andreas R. Gschwind [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0769-6907>),
  Lars Velten [aut] (ORCID: <https://orcid.org/0000-0002-1233-5874>),
  Lars M. Steinmetz [aut]
Maintainer: Andreas R. Gschwind <andreas.gschwind@stanford.edu>
