# Generated by roxygen2: do not edit by hand

export("beads_oligo<-")
export("min_primer_region<-")
export("primer_max_tm<-")
export("primer_min_tm<-")
export("primer_num_return<-")
export("primer_opt_tm<-")
export("product_size_range<-")
export("reverse_primer<-")
export("sequence_id<-")
export("target_annot<-")
export("target_sequence<-")
export(TAPseqInput)
export(TsIO)
export(TsIOList)
export(beads_oligo)
export(blastPrimers)
export(checkPrimers)
export(createBLASTDb)
export(createIORecord)
export(createPrimerTrack)
export(designPrimers)
export(exportPrimerTrack)
export(getTxsSeq)
export(inferPolyASites)
export(min_primer_region)
export(pcr_products)
export(pickPrimers)
export(plotTargetGenes)
export(primerDataFrame)
export(primer_max_tm)
export(primer_min_tm)
export(primer_num_return)
export(primer_opt_tm)
export(product_size_range)
export(reverse_primer)
export(selectTargetGenes)
export(sequence_id)
export(sequence_template)
export(tapseq_primers)
export(target_annot)
export(target_sequence)
export(truncateTxsPolyA)
exportMethods("beads_oligo<-")
exportMethods("min_primer_region<-")
exportMethods("primer_max_tm<-")
exportMethods("primer_min_tm<-")
exportMethods("primer_num_return<-")
exportMethods("primer_opt_tm<-")
exportMethods("product_size_range<-")
exportMethods("reverse_primer<-")
exportMethods("sequence_id<-")
exportMethods("target_annot<-")
exportMethods("target_sequence<-")
exportMethods(beads_oligo)
exportMethods(blastPrimers)
exportMethods(checkPrimers)
exportMethods(createIORecord)
exportMethods(createPrimerTrack)
exportMethods(designPrimers)
exportMethods(getTxsSeq)
exportMethods(min_primer_region)
exportMethods(pcr_products)
exportMethods(pickPrimers)
exportMethods(primerDataFrame)
exportMethods(primer_max_tm)
exportMethods(primer_min_tm)
exportMethods(primer_num_return)
exportMethods(primer_opt_tm)
exportMethods(product_size_range)
exportMethods(reverse_primer)
exportMethods(sequence_id)
exportMethods(sequence_template)
exportMethods(tapseq_primers)
exportMethods(target_annot)
exportMethods(target_sequence)
exportMethods(truncateTxsPolyA)
import(BSgenome)
import(BiocGenerics)
import(GenomicRanges)
import(methods)
importClassesFrom(Biostrings,DNAString)
importClassesFrom(Biostrings,DNAStringSet)
importClassesFrom(IRanges,IRanges)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,end)
importFrom(Biostrings,matchPattern)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,start)
importFrom(Biostrings,subseq)
importFrom(GenomicFeatures,extractTranscriptSeqs)
importFrom(IRanges,IRanges)
importFrom(IRanges,IRangesList)
importFrom(IRanges,Views)
importFrom(IRanges,ranges)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,endoapply)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,mendoapply)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,split)
importFrom(S4Vectors,subjectHits)
importFrom(dplyr,bind_rows)
importFrom(dplyr,distinct)
importFrom(dplyr,left_join)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,separate)
